/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*global define, history, document, console */

/**
 * Application module.
 *
 * @module app
 * @requires {@link core/event}
 * @requires {@link core/application}
 * @requires {@link core/systeminfo}
 * @requires {@link views/main}
 * @requires {@link views/details}
 * @namespace app
 */

define({
    name: 'app',
    requires: [
        'core/event',
        'core/application',
        'core/systeminfo',
        'views/main',
        'views/details'
    ],
    def: function initApp(req) {
        'use strict';

        /**
         * Event core module.
         *
         * @private
         * @type {Module}
         */
        var e = req.core.event,

            /**
             * Application core module.
             *
             * @private
             * @type {Module}
             */
            app = req.core.application,

            /**
             * System information core module.
             *
             * @private
             * @type {Module}
             */
            sysInfo = req.core.systeminfo;

        /**
         * Registers event listeners.
         *
         * @private
         */
        function bindEvents() {
            document.addEventListener('tizenhwkey', function onTizenhwkey(e) {
                if (e.keyName === 'back') {
                    if (document.getElementsByClassName('ui-page-active')[0]
                            .id === 'main') {
                        app.exit();
                    } else {
                        history.back();
                    }
                }
            });

            try {
                sysInfo.listenBatteryLowState();
            } catch (error) {
                console.warn('battery state listener error', error);
            }
        }

        /**
         * Handles low battery state.
         *
         * @private
         */
        function onLowBattery() {
            app.exit();
        }

        /**
         * Initializes module.
         *
         * @memberof app
         * @public
         */
        function init() {
            bindEvents();
            try {
                sysInfo.checkBatteryLowState();
            } catch (error) {
                console.warn('battery check error', error);
            }

            e.listeners({
                'core.systeminfo.battery.low': onLowBattery
            });
        }

        return {
            init: init
        };
    }
});
